# -*- encoding:utf-8 -*-
from __future__ import unicode_literals

MESSAGES = {
    "Also available in": "Ook beschikbaar in",
    "Archive": "Archief",
    "Categories": "Categorieën",
    "LANGUAGE": "Nederlands",
    "More posts about": "Meer berichten over",
    "Newer posts": "Nieuwere berichten",
    "Next post": "Volgend bericht",
    "Older posts": "Oudere berichten",
    "Original site": "Originele site",
    "Posted": "Geplaatst",
    "Posts about %s": "Berichten over %s",
    "Posts for year %s": "Berichten voor het jaar %s",
    "Posts for {month} {year}": "Berichten voor {month} {year}",
    "Previous post": "Vorig bericht",
    "Read in English": "Lees in het Nederlands",
    "Read more": "Lees verder",
    "Source": "Bron",
    "Tags and Categories": "Tags en Categorieën",
    "Tags": "Tags",
    "old posts page %d": "oude berichten pagina %d",
}
