# -*- encoding:utf-8 -*-
from __future__ import unicode_literals

MESSAGES = {
    "Also available in": "Također dostupno i u",
    "Archive": "Arhiva",
    "Categories": "",
    "LANGUAGE": "hrvatski",
    "More posts about": "Više postova o",
    "Newer posts": "Noviji postovi",
    "Next post": "Sljedeći post",
    "Older posts": "Stariji postovi",
    "Original site": "Izvorna stranica",
    "Posted": "Objavljeno",
    "Posts about %s": "Postovi o %s",
    "Posts for year %s": "Postovi za godinu %s",
    "Posts for {month} {year}": "Postovi za {month} {year}",
    "Previous post": "Prethodni post",
    "Read in English": "Čitaj na hrvatskom",
    "Read more": "Čitaj dalje",
    "Source": "Izvor",
    "Tags and Categories": "",
    "Tags": "Tagovi",
    "old posts page %d": "stari postovi stranice %d",
}
