# -*- encoding:utf-8 -*-
from __future__ import unicode_literals

MESSAGES = {
    "Also available in": "Ankaŭ disponebla en",
    "Archive": "Arĥivo",
    "Categories": "Kategorioj",
    "LANGUAGE": "Anglalingve",
    "More posts about": "Pli artikoloj pri...",
    "Newer posts": "Pli novaj artikoloj",
    "Next post": "Venonta artikolo",
    "Older posts": "Pli malnovaj artikoloj",
    "Original site": "Originala interretejo",
    "Posted": "Skribita",
    "Posts about %s": "Artikoloj pri %s",
    "Posts for year %s": "Artikoloj de la jaro %s",
    "Posts for {month} {year}": "Artikoloj skribitaj en {month} {year}",
    "Previous post": "Antaŭa artikolo",
    "Read in English": "Legu ĝin en Esperanto",
    "Read more": "Legu plu",
    "Source": "Fonto",
    "Tags and Categories": "Etikedoj kaj Kategorioj",
    "Tags": "Etikedoj",
    "old posts page %d": "paĝo de malnovaj artikoloj %d",
}
